<?php

namespace Drupal\expay\Events;

use Drupal\expay\Provider\ExpayProvider;
use Symfony\Component\HttpFoundation\JsonResponse;

/** Class Base Event for all Expay events */
abstract class ExpayBaseEvent {
  protected $payment;
  protected $provider;

  public function __construct($payment) {
    $this->provider = new ExpayProvider();
    $this->payment = $payment;
  }

  public abstract function handle($event);

    /**
     * @param $code
     * @param $message
     * @return JsonResponse
     */
  protected function sendResponse($code, $message) : JsonResponse {
    $response = [
      'response' => [
        'status' => $code,
        'message' => $message,
        'timestamp' => time()
      ]
    ];
    $response['hash'] = $this->provider->hashResponse($response);

    return new JsonResponse($response, $code);
  }

    /**
     * @param $event
     * @return bool
     */
  protected function isOrderValid($event) : bool {
    $order = $this->payment->getOrder();
    if (!($isInvalid = isset($order))) {
      \Drupal::logger('expay')->alert('Expay Webhooks: the order#' . $event['id'] . 'doesn\'t exist');
    }
    return $isInvalid;
  }

  protected function isOrderCanceled() : bool {
    $order = $this->payment->getOrder();
    $state = $order->getState()->getValue();
    if ($isCanceled = ($state['value'] === 'canceled')) {
      \Drupal::logger('expay')->alert('Expay Webhooks: the order#' . $order->id() . ' has already been canceled');
    }
    return $isCanceled;
  }

  protected function isOrderPaid() : bool {
    $state = $this->payment->getOrder()->getState()->getValue();
    return $state['value'] === 'completed';
  }

  protected function isHashValid($event) : bool {
    if (!isset($event['hash'])) {
      \Drupal::logger('expay')->alert('Expay Webhooks: invalid hash');
      return false;
    }

    $hash = $event['hash'];
    unset($event['hash']);
    if ($hash !== $this->provider->hashResponse($event)) {
      \Drupal::logger('expay')->alert('Expay Webhooks: invalid hash');
      return false;
    }
    return true;
  }

  protected function amountMatching($event) : bool {
    $order = $this->payment->getOrder();
    $amount = sprintf("%.2f", $order->getTotalPrice()->getNumber());
    if ($amountNotMatches = ($amount !== $event['amount'])) {
      \Drupal::logger('expay')->alert('Expay Webhooks: The amount of order ' . $order->id() . " doesn't match.");
    }
    return !$amountNotMatches;
  }
}